defmodule Gh2ntfy.GhPoll do
  alias Gh2ntfy.LastPoll
  alias Gh2ntfy.Util

  require Logger

  use GenServer

  def start_link(token) when is_binary(token) do
    GenServer.start_link(__MODULE__, token, name: __MODULE__)
  end

  @impl true
  def init(token) do
    Process.send(self(), :poll, [])
    {:ok, token}
  end

  @impl true
  def handle_info(:poll, token) do
    schedule()

    Logger.info("polling notifications")

    result =
      Finch.build(
        :get,
        "https://api.github.com/notifications?since=#{LastPoll.get_as_iso8601()}",
        [
          {"Authorization", "Bearer " <> token},
          {"Time-Zone", "Etc/UTC"}
        ]
      )
      |> Finch.request(Gh2ntfy.Finch)

    case result do
      {:ok, %{body: body}} ->
        case Util.catch_tuple(fn -> :jiffy.decode(body, [:return_maps]) end) do
          {:ok, res} ->
            Gh2ntfy.LastPoll.set_now()
            process_response(res)

          {:error, e} ->
            Logger.error("invalid json from github: " <> Exception.format(:error, e))
        end

      {:error, e} ->
        Logger.error("requesting info: " <> Exception.format(:error, e))
    end

    {:noreply, token}
  end

  defp process_response(res) do
    Enum.each(res, fn notif ->
      case notif do
        %{
          "repository" => %{"full_name" => title},
          "subject" => %{"title" => body, "latest_comment_url" => cmt_url},
          "reason" => reason
        } ->
          url_headers =
            try do
              [{"Click", get_issuecomment_url(cmt_url)}]
            rescue
              e ->
                Logger.warn("fetching issuecomment URL: " <> Exception.format(:error, e))
                []
            end

          Gh2ntfy.Ntfyer.send_notif("GitHub: " <> title, "#{reason}\n#{body}", url_headers)

        _ ->
          Logger.warn("invalid data from github")
          nil
      end
    end)
  end

  defp schedule do
    Process.send_after(self(), :poll, :timer.minutes(30))
  end

  defp get_issuecomment_url(api_url) do
    {:ok, %{body: body}} = Finch.build(:get, api_url) |> Finch.request(Gh2ntfy.Finch)
    %{"html_url" => url} = :jiffy.decode(body, [:return_maps])
    url
  end
end
